/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2001

   Test-suite for branch-level Access Control Lists

*/

#include "branch-acl.h"
#include "check_acl.h"
#include "dir-acl.h"

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>


void
check_nonexisting_file_acl (char *top_dir)
{
    DIR_ACL *topdir_acl;
    BRANCH_ACL *trunk_acl;

    /* check that ACLs don't work for non-existing file */
    topdir_acl = get_dir_acl(top_dir, "trunk");
    assert(topdir_acl != NULL);

    trunk_acl = get_branch_acl(topdir_acl, "trunk");
    assert(trunk_acl != NULL);
}


void
check_reading_acl_file (void)
{
/*
    assert(internal_read_branch_acl_file("./test-acl-non-existent.txt", "trunk", NULL) == 0);

    assert((internal_read_branch_acl_file("./test-acl-non-readable.txt", "trunk", NULL) == -1) && (errno == EACCES));

    assert(internal_read_branch_acl_file("./test-acl-too-long-line.txt", "trunk", NULL) == 1);

    assert(internal_read_branch_acl_file("./test-acl-too-few.txt", "trunk", NULL) == 1);
    assert(internal_read_branch_acl_file("./test-acl-too-many.txt", "trunk", NULL) == 1);
*/
}


void
check_existing_file_acl (char *top_dir)
{
    DIR_ACL *topdir_acl;
    BRANCH_ACL *trunk_acl;
    BRANCH_ACE *foo_ace;
    int new_perm;
    char trunk[] = "trunk";

    topdir_acl = get_dir_acl(top_dir, "trunk");
    assert(topdir_acl != NULL);

    /* get list of permissions for trunk, it should be empty */
    trunk_acl = get_branch_acl(topdir_acl, trunk);
    assert(trunk_acl != NULL);
    assert(trunk_acl->branch != NULL);
    assert(strcmp(trunk_acl->branch, trunk) == 0);
    assert(trunk_acl->dir_acl == topdir_acl);
    assert(trunk_acl->first_ace == NULL);

    /* check that trunk_acl has its own copy of branch name */
    strcpy(trunk, "knurt");
    assert(strcmp(trunk_acl->branch, "trunk") == 0);

    /* get list of permissions for file, it should be default */
    foo_ace = get_file_on_branch_ace(trunk_acl, "foo");
    assert(foo_ace != NULL);
    assert(foo_ace->acl == trunk_acl);
    assert(foo_ace->fullpath != NULL);
    assert(foo_ace->filename != NULL);
    assert(strcmp(foo_ace->filename->buf, "foo") == 0);
    assert(foo_ace->user_acls == NULL);
    assert(foo_ace->next == NULL);

    /* default permissions allow to checkin/checkout revision on any branch */
    assert(branch_permission_granted(foo_ace, "vasya", branch_perm_checkin));
    assert(branch_permission_granted(foo_ace, "vasya", branch_perm_checkout));

    /* set only checkout permissions for vasya, check it */
    new_perm = set_branch_ace(foo_ace, "vasya", branch_perm_checkout);
    assert(new_perm == branch_perm_checkout);

    assert(branch_permission_granted(foo_ace, "vasya", branch_perm_checkout));

    assert(!branch_permission_granted(foo_ace, "vasya", branch_perm_checkin));

    /* set only checkin permissions for vasya, check it */
    new_perm = set_branch_ace(foo_ace, "vasya", branch_perm_checkin);
    assert(new_perm == branch_perm_checkin);

    assert(!branch_permission_granted(foo_ace, "vasya", branch_perm_checkout));

    assert(branch_permission_granted(foo_ace, "vasya", branch_perm_checkin));

    /* check that petya cannot get to file with ACLs */
    assert(!branch_permission_granted(foo_ace, "petya", branch_perm_checkout));

    assert(!branch_permission_granted(foo_ace, "petya", branch_perm_checkin));

    /* set no permissions for vasya, check it */
    new_perm = set_branch_ace(foo_ace, "vasya", branch_perm_none);
    assert(new_perm == branch_perm_none);

    assert(!branch_permission_granted(foo_ace, "vasya", branch_perm_checkout));

    assert(!branch_permission_granted(foo_ace, "vasya", branch_perm_checkin));

    /* set checkout permissions again */
    new_perm = set_branch_ace(foo_ace, "vasya", branch_perm_checkout);
    assert(new_perm == branch_perm_checkout);

    /* remove vasya from file ACLs */
    new_perm = set_branch_ace(foo_ace, "vasya", branch_perm_none);
    assert (new_perm == branch_perm_none);

    assert(!branch_permission_granted(foo_ace, "vasya", branch_perm_checkout));
}

#define INTRO \
\
    DIR_ACL *topdir_acl; \
    BRANCH_ACL *trunk_acl; \
    BRANCH_ACL *branch_acl; \
    BRANCH_ACE *foo_on_branch_ace; \
    BRANCH_ACE *foo_on_trunk_ace; \
\
    topdir_acl = get_dir_acl("./test-acl", "BRANCH"); \
    assert(topdir_acl != NULL); \
\
    trunk_acl = get_branch_acl(topdir_acl, "trunk"); \
    assert(trunk_acl != NULL); \
\
    branch_acl = get_branch_acl(topdir_acl, "BRANCH"); \
    assert(branch_acl != NULL); \
\
    foo_on_branch_ace = get_file_on_branch_ace(branch_acl, "foo"); \
    assert(foo_on_branch_ace != NULL); \
\
    foo_on_trunk_ace = get_file_on_branch_ace(trunk_acl, "foo"); \
    assert(foo_on_trunk_ace != NULL); \




void
check_on_branch_acl (void)
{
    INTRO;

    /* try to set permissions on trunk and see if they are used on
       branch only if not re-defined on branch */

    assert(set_branch_ace(foo_on_trunk_ace, "alice", branch_perm_checkin) == branch_perm_checkin);

    /* check that permissions on trunk are used as default permissions on branches */
    assert(branch_permission_granted(foo_on_branch_ace, "alice", branch_perm_checkin));
    assert(!branch_permission_granted(foo_on_branch_ace, "alice", branch_perm_checkout));

}


void
check_default_dir_on_branch_acl (void)
{
    INTRO;

    /* set default permissions on trunk and on branch */
    assert(set_default_file_acl(topdir_acl, "trunk", "alice", branch_perm_checkout) == branch_perm_checkout);

    assert(set_default_file_acl(topdir_acl, "BRANCH", "alice", branch_perm_none) == branch_perm_none);

    /* ...and see if they work independently */
    assert(branch_permission_granted(foo_on_trunk_ace, "alice", branch_perm_checkout));
    assert(!branch_permission_granted(foo_on_trunk_ace, "alice", branch_perm_checkin));

}

void
check_persistent_on_branch_acl (char *topdir)
{
    INTRO;

    assert(branch_permission_granted(foo_on_branch_ace, "alice", branch_perm_checkin));
    assert(branch_permission_granted(foo_on_branch_ace, "alice", branch_perm_checkout));

    assert(branch_permission_granted(foo_on_trunk_ace, "alice", branch_perm_checkin));
    assert(branch_permission_granted(foo_on_trunk_ace, "alice", branch_perm_checkout));


    /* set permissions on trunk; change permissions on branch: they
       should remain unchanged on trunk */
    set_branch_ace(foo_on_trunk_ace, "alice", branch_perm_checkin);
    set_branch_ace(foo_on_branch_ace, "alice", branch_perm_none);

    assert(branch_permission_granted(foo_on_trunk_ace, "alice", branch_perm_checkin));
    assert(!branch_permission_granted(foo_on_trunk_ace, "alice", branch_perm_checkout));
    assert(!branch_permission_granted(foo_on_branch_ace, "alice", branch_perm_checkin));
    assert(!branch_permission_granted(foo_on_branch_ace, "alice", branch_perm_checkout));

    /* check that permissions are persistent */
    assert(store_branch_acls(topdir_acl));

    topdir_acl = get_dir_acl(topdir, "BRANCH");
    assert(topdir_acl != NULL);

    branch_acl = get_branch_acl(topdir_acl, "BRANCH");
    assert(branch_acl != NULL);

    foo_on_branch_ace = get_file_on_branch_ace(branch_acl, "foo");
    assert(foo_on_branch_ace != NULL);

    trunk_acl = get_branch_acl(topdir_acl, "trunk");
    assert(trunk_acl != NULL);

    foo_on_trunk_ace = get_file_on_branch_ace(trunk_acl, "foo");
    assert(foo_on_trunk_ace != NULL);

    assert(branch_permission_granted(foo_on_trunk_ace, "alice", branch_perm_checkin));
    assert(!branch_permission_granted(foo_on_trunk_ace, "alice", branch_perm_checkout));
    assert(!branch_permission_granted(foo_on_branch_ace, "alice", branch_perm_checkin));
    assert(!branch_permission_granted(foo_on_branch_ace, "alice", branch_perm_checkout));
}

void
check_parsing_branch_permissions (void)
{
    assert(parse_branch_permissions("invalid") == branch_perm_invalid);

    assert(parse_branch_permissions("none") == branch_perm_none);

    assert(parse_branch_permissions("checkin") == branch_perm_checkin);

    assert(parse_branch_permissions("checkout") == branch_perm_checkout);

    assert(parse_branch_permissions("checkin,checkout") == 
	   (branch_perm_checkin | branch_perm_checkout));

    assert(parse_branch_permissions("checkout,checkin") == 
	   (branch_perm_checkin | branch_perm_checkout));


    assert(strcmp(branch_permissions_string(branch_perm_checkin), "checkin") == 0);

    assert(strcmp(branch_permissions_string(branch_perm_checkout), "checkout") == 0);

    assert(strcmp(branch_permissions_string(branch_perm_none), "none") == 0);

    assert(strcmp(branch_permissions_string(branch_perm_checkin | branch_perm_checkout),
		  "checkin,checkout") == 0);
}
