
#include "check_acl.h"
#include "dir-acl.h"
#include "branch-acl.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

static void
check_default_file_acl_internals (void)
{
    struct DIR_ACL *topdir_acl;
    struct DEFAULT_FILE_ACL *dfa;

    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl != NULL);

    assert(topdir_acl->default_file_acl == NULL);

    assert(set_default_file_acl(topdir_acl, "BRANCH", "bob", branch_perm_all) == branch_perm_all);
    dfa = topdir_acl->default_file_acl;
    assert(dfa != NULL);
    assert(strcmp(dfa->branch, "BRANCH") == 0);
    assert(dfa->user_acl != NULL);
    assert(dfa->user_acl->next == NULL);
    assert(strcmp(dfa->user_acl->username, "bob") == 0);
    assert(dfa->user_acl->perm == branch_perm_all);
}

static void
check_default_file_acl (void)
{
    struct DIR_ACL *topdir_acl;
    struct BRANCH_ACL *trunk_acl;
    struct BRANCH_ACE *foo_ace;

    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl != NULL);

    /* we set the default file permissions */
    assert(set_default_file_acl(topdir_acl, "trunk", "alice", branch_perm_checkout)
	   == branch_perm_checkout);

    trunk_acl = get_branch_acl(topdir_acl, "trunk");
    assert(trunk_acl != NULL);

    foo_ace = get_file_on_branch_ace(trunk_acl, "foo");
    assert(foo_ace != NULL);

    /* check that newly set default file permissions are honoured */
    assert(branch_permission_granted(foo_ace, "alice", branch_perm_checkout));
    assert(!branch_permission_granted(foo_ace, "alice", branch_perm_checkin));

    /* check that default file permissions are honoured */
    assert(branch_permission_granted(foo_ace, "bob", branch_perm_checkout));
    assert(branch_permission_granted(foo_ace, "bob", branch_perm_checkin));
}


static void
check_persistent_default_file_acl (void)
{
    struct DIR_ACL *topdir_acl;
    struct BRANCH_ACL *trunk_acl;
    struct BRANCH_ACE *foo_ace;

    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl);

    /* we set the default file permissions... */
    assert(set_default_file_acl(topdir_acl, "trunk", "alice", branch_perm_checkin)
	   == branch_perm_checkin);

    /* ... store them to disk ... */
    if (store_dir_acl(topdir_acl) != 0) {
	perror("Storing dir ACL");
	exit(1);
    }

    /* ... and read them back */
    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl != NULL);

    trunk_acl = get_branch_acl(topdir_acl, "trunk");
    assert(trunk_acl != NULL);

    foo_ace = get_file_on_branch_ace(trunk_acl, "foo");
    
    /* check that newly set default file permissions are honoured */
    assert(branch_permission_granted(foo_ace, "alice", branch_perm_checkin));
    assert(!branch_permission_granted(foo_ace, "alice", branch_perm_checkout));

    /* check that default file permissions are honoured */
    assert(branch_permission_granted(foo_ace, "bob", branch_perm_checkout));
    assert(branch_permission_granted(foo_ace, "bob", branch_perm_checkin));
}


static void
check_default_file_acl_in_subdirs (void)
{

    struct DIR_ACL *sdir_acl;
    struct DIR_ACL *ssdir_acl;

    struct BRANCH_ACL *trunk_acl;

    struct BRANCH_ACE *bar_ace;
    struct BRANCH_ACE *baz_ace;

    sdir_acl = get_dir_acl("./test-acl/sdir", "trunk");
    assert(sdir_acl != NULL);

    ssdir_acl = get_dir_acl("./test-acl/sdir/ssdir", "trunk");
    assert(ssdir_acl != NULL);


    /* check that default file permissions are the same as in
       top-level directory */
    trunk_acl = get_branch_acl(sdir_acl, "trunk");
    assert(trunk_acl != NULL);

    bar_ace = get_file_on_branch_ace(trunk_acl, "bar");
    assert(bar_ace != NULL);

    /* check that newly set default file permissions are honoured */
    assert(branch_permission_granted(bar_ace, "alice", branch_perm_checkin));
    assert(!branch_permission_granted(bar_ace, "alice", branch_perm_checkout));

    /* check that default file permissions are honoured */
    assert(branch_permission_granted(bar_ace, "bob", branch_perm_checkout));
    assert(branch_permission_granted(bar_ace, "bob", branch_perm_checkin));

    trunk_acl = get_branch_acl(ssdir_acl, "trunk");
    assert(trunk_acl != NULL);

    baz_ace = get_file_on_branch_ace(trunk_acl, "baz");
    assert(bar_ace != NULL);

    /* check that newly set default file permissions are honoured */
    assert(branch_permission_granted(baz_ace, "alice", branch_perm_checkin));
    assert(!branch_permission_granted(baz_ace, "alice", branch_perm_checkout));

    /* check that default file permissions are honoured */
    assert(branch_permission_granted(baz_ace, "bob", branch_perm_checkout));
    assert(branch_permission_granted(baz_ace, "bob", branch_perm_checkin));

}


int main (void)
{
    check_default_file_acl_internals();

    check_default_file_acl();

    check_persistent_default_file_acl();

    check_default_file_acl_in_subdirs();

    exit(0);
}
