/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2001-2002

   Test-suite for directory-level Access Control Lists

*/

#include "config.h"

#include "check_acl.h"

#include "dir-acl.h"
#include "user-acl.h"

#include <assert.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>


static void
check_perm_parsing (void)
{
    assert(parse_dir_permissions("INVALID") == dir_perm_invalid);
    assert(parse_dir_permissions("none") == dir_perm_none);
    assert(parse_dir_permissions("access") == dir_perm_access);
    assert(parse_dir_permissions("modify") == dir_perm_modify);
    assert(parse_dir_permissions("access,modify") == dir_perm_all);
    assert(parse_dir_permissions("modify,access") == dir_perm_all);

    assert(strcmp(dir_permissions_string(dir_perm_none), "none") == 0);
    assert(strcmp(dir_permissions_string(dir_perm_access), "access") == 0);
    assert(strcmp(dir_permissions_string(dir_perm_modify), "modify") == 0);
    assert(strcmp(dir_permissions_string(dir_perm_all), "access,modify") == 0);
}

static void
check_nonexisting_dir_acl (void)
{
    DIR_ACL *topdir_acl;

    /* check that ACLs don't work for non-existing directory */
    errno = 0;
    topdir_acl = get_dir_acl("./nosuchdir", "trunk");
    assert(topdir_acl == NULL);
    assert(errno == ENOENT);
}


static void
check_default_dir_permissions (void)
{
    DIR_ACL *topdir_acl;

    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl != NULL);

    /* check that default permissions (when the directory ACL is
       empty) allow everything */
    assert(dir_permission_granted(topdir_acl, "vasya", dir_perm_access));
    assert(dir_permission_granted(topdir_acl, "vasya", dir_perm_modify));

    /* set default dir permissions to none */
    set_default_dir_perm(dir_perm_none);
    assert(!dir_permission_granted(topdir_acl, "vasya", dir_perm_access));
    assert(!dir_permission_granted(topdir_acl, "vasya", dir_perm_modify));

    /* set default dir permissions back to all */
    set_default_dir_perm(dir_perm_all);
}

static void
check_nonpersistent_dir_acl (void)
{
    DIR_ACL *topdir_acl;

    /* check that default ACLs get created for existing directory,
       even when there is no DIRACL for that directory */
    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl != NULL);
    assert(topdir_acl->user_acls == NULL);

    /* try to set the permissions on directory and see if it works */
    assert(set_dir_ace(topdir_acl, "vasya", dir_perm_access));

    assert(dir_permission_granted(topdir_acl, "vasya", dir_perm_access));
    assert(!dir_permission_granted(topdir_acl, "vasya", dir_perm_modify));

    /* Check that when there is at least one user mentioned in a
       directory ACL, all other users get permissions according to
       repository policy */
    set_default_dir_perm(dir_perm_none);
    assert(!dir_permission_granted(topdir_acl, "mike", dir_perm_access));
    assert(!dir_permission_granted(topdir_acl, "mike", dir_perm_modify));

    set_default_dir_perm(dir_perm_all);
    assert(dir_permission_granted(topdir_acl, "mike", dir_perm_access));
    assert(dir_permission_granted(topdir_acl, "mike", dir_perm_modify));
}

static void
check_persistent_dir_acl (void)
{
    struct DIR_ACL *topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl);
    assert(!topdir_acl->branch_acls_loaded);

    assert(set_dir_ace(topdir_acl, "alice", dir_perm_access) == dir_perm_access);
    assert(set_dir_ace(topdir_acl, "bob", dir_perm_modify) == dir_perm_modify);
    assert(set_dir_ace(topdir_acl, "dave", dir_perm_all) == dir_perm_all);

    if (store_dir_acl(topdir_acl) != 0) {
	perror("Storing dir ACL");
	exit(1);
    }

    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl);
    assert(!topdir_acl->branch_acls_loaded);

    /* alice gets only access permissions */
    assert(dir_permission_granted(topdir_acl, "alice", dir_perm_access));
    assert(!dir_permission_granted(topdir_acl, "alice", dir_perm_modify));

    /* bob gets only modify permissions */
    assert(!dir_permission_granted(topdir_acl, "bob", dir_perm_access));
    assert(dir_permission_granted(topdir_acl, "bob", dir_perm_modify));

    /* carol doesn't get permissions, because she isn't in the explicit list of ACLs */
#if 0
    assert(!dir_permission_granted(topdir_acl, "carol", dir_perm_access));
    assert(!dir_permission_granted(topdir_acl, "carol", dir_perm_modify));
#endif

    /* dave gets both access and modify permissions */
    assert(dir_permission_granted(topdir_acl, "dave", dir_perm_access));
    assert(dir_permission_granted(topdir_acl, "dave", dir_perm_modify));

    /* now revoke bob's permissions and check that it persists */
    assert(set_dir_ace(topdir_acl, "bob", dir_perm_none) == dir_perm_none);
    
    if (store_dir_acl(topdir_acl) != 0) {
	perror("Storing dir ACL");
	exit(1);
    }

    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl);

    assert(!dir_permission_granted(topdir_acl, "bob", dir_perm_access));
    assert(!dir_permission_granted(topdir_acl, "bob", dir_perm_modify));
}

int
main (void)
{
    check_perm_parsing();

    check_nonexisting_dir_acl();

    check_default_dir_permissions();

    check_nonpersistent_dir_acl();

    check_persistent_dir_acl();

    exit(0);
}
