/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2001

   Test-suite for module-level Access Control Lists

*/

#include "module-acl.h"

#include <assert.h>
#include <stdlib.h>


static void
check_persistent_acl (void)
{
    struct REPOSITORY *repos;
    struct MODULE_ACL *mod1_acl;
    struct MODULE_ACL *mod2_acl;

    repos = get_repository("./repos");
    assert(repos != NULL);

    mod1_acl = get_module_acl(repos, "module-foo");
    assert(mod1_acl != NULL);

    mod2_acl = get_module_acl(repos, "module-bar");
    assert(mod2_acl != NULL);

    /* grant some permissions */
    assert(grant_module_tagging(mod1_acl, "alice", "RELEASE"));
    assert(grant_module_tagging(mod1_acl, "bob", "TESTING"));

    assert(grant_module_tagging(mod2_acl, "bob", "RELEASE"));

    assert(grant_module_branching(mod1_acl, "alice", "PATCHES"));
    assert(grant_module_branching(mod2_acl, "bob", "EXP"));

    /* check that saving permissions works */
    assert(save_module_acl(repos));

    free_repository(repos);

    repos = get_repository("./repos");
    assert(repos != NULL);

    /* check that saved permissions still work */
    mod1_acl = get_module_acl(repos, "module-foo");
    assert(mod1_acl != NULL);

    mod2_acl = get_module_acl(repos, "module-bar");
    assert(mod2_acl != NULL);

    assert(module_tagging_granted(mod1_acl, "alice", "RELEASE-1"));
    assert(!module_tagging_granted(mod1_acl, "alice", "FORBIDDEN-TAG"));

    assert(module_branching_granted(mod1_acl, "alice", "PATCHES-1-2-3"));
    assert(!module_branching_granted(mod1_acl, "bob", "PATCHES-10"));

    assert(module_tagging_granted(mod1_acl, "bob", "TESTING-2"));
    assert(!module_tagging_granted(mod2_acl, "bob", "TESTING-2"));

    assert(module_tagging_granted(mod2_acl, "bob", "RELEASE-10"));

    assert(module_branching_granted(mod2_acl, "bob", "EXP-2"));
    assert(!module_branching_granted(mod2_acl, "bob", "PATCHES-8"));

    mod2_acl = get_module_acl(repos, "module-bar");
    assert(mod2_acl != NULL);

    /* check that revoking permissions works */
    assert(revoke_module_tagging(mod1_acl, "alice", "RELEASE"));

    assert(!module_tagging_granted(mod1_acl, "alice", "RELEASE-1"));

    free_repository(repos);
}

int
main (void)
{
    struct REPOSITORY *repos;
    struct MODULE_ACL *mod1_acl;

    repos = get_repository("./repos");
    assert(repos != NULL);

    /* check low-level simple stuff */
    mod1_acl = get_module_acl(repos, "module-1");
    assert(mod1_acl != NULL);
    assert(mod1_acl->repository != NULL);
    assert(mod1_acl->repository == repos);
    assert(strcmp(mod1_acl->module, "module-1") == 0);
    assert(mod1_acl->tag_acl == NULL);

    /* check that default tagging permissions work */
    assert(module_tagging_granted(mod1_acl, "alice", "RELEASE-1"));

    assert(module_branching_granted(mod1_acl, "alice", "BRANCH-1"));

    /* check that granting permissions works */
    assert(grant_module_tagging(mod1_acl, "alice", "RELEASE"));
    assert(grant_module_branching(mod1_acl, "alice", "PATCHES-"));

    /* creating branches and tags allowed */
    assert(module_branching_granted(mod1_acl, "alice", "PATCHES-10"));
    assert(module_tagging_granted(mod1_acl, "alice", "RELEASE-5-0"));

    /* forbidden branches and tags are forbidden */
    assert(!module_tagging_granted(mod1_acl, "alice", "FORBIDDEN-1"));
    assert(!module_branching_granted(mod1_acl, "alice", "FIXED-10"));

    /* branches are not mistaken with tags */
    assert(!module_branching_granted(mod1_acl, "alice", "RELEASE-10"));
    assert(!module_tagging_granted(mod1_acl, "alice", "PATCHES-1-2-3"));

    free_repository(repos);

    check_persistent_acl();

    exit(0);
}
