/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2001

   Test-suite for branch-level Access Control Lists stored to file

*/


#include "config.h"

#include "dir-acl.h"
#include "branch-acl.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

static void
check_single_persistent_file_acl (void)
{
    DIR_ACL *topdir_acl;
    BRANCH_ACL *trunk_acl;
    BRANCH_ACE *foo_ace;
    BRANCH_ACE *foo_ace2;
    BRANCH_ACE *bar_ace;
    int new_perm;

    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl != NULL);

    /* get default permissions for existing file */
    trunk_acl = get_branch_acl(topdir_acl, "trunk");

    /* get ACE for file, causing it to be created */
    foo_ace = get_file_on_branch_ace(trunk_acl, "foo");
    assert(foo_ace != NULL);

    /* set checkout permissions for vasya and save it to disk */
    new_perm = set_branch_ace(foo_ace, "vasya", branch_perm_checkout);
    new_perm = set_branch_ace(foo_ace, "petya", branch_perm_checkin);

    if (!store_branch_acls(topdir_acl)) {
	perror("Storing branch ACLs");
	exit(1);
    }

    trunk_acl = get_branch_acl (topdir_acl, "trunk");

    /* read those permissions from disk and check them */
    foo_ace2 = get_file_on_branch_ace(trunk_acl, "foo");
    assert(foo_ace2 != NULL);
    assert(foo_ace2->user_acls != NULL);
    assert(foo_ace2->fullpath != NULL);
    assert(foo_ace2->filename != NULL);
    assert(strcmp(foo_ace2->filename->buf, "foo") == 0);
    assert(foo_ace2->acl == trunk_acl);

    assert(branch_permission_granted(foo_ace2, "vasya", branch_perm_checkout));
    assert(!branch_permission_granted(foo_ace2, "vasya", branch_perm_checkin));

    assert(!branch_permission_granted(foo_ace2, "petya", branch_perm_checkout));
    assert(branch_permission_granted(foo_ace2, "petya", branch_perm_checkin));


    /* check that permissions for another file have not changed and
       are, in fact, default ones */
    bar_ace = get_file_on_branch_ace(trunk_acl, "bar");
    assert(bar_ace != NULL);

    assert(branch_permission_granted(bar_ace, "vasya", branch_perm_checkout));
    assert(branch_permission_granted(bar_ace, "vasya", branch_perm_checkin));
    assert(branch_permission_granted(bar_ace, "petya", branch_perm_checkout));
    assert(branch_permission_granted(bar_ace, "petya", branch_perm_checkin));
}


static void
check_several_persistent_files_acl (void)
{
    DIR_ACL *dir_acl;
    BRANCH_ACL *trunk_acl;
    BRANCH_ACE *foo_ace;
    BRANCH_ACE *bar_ace;

    dir_acl = get_dir_acl("./test-acl", "trunk");
    assert(dir_acl != NULL);

    trunk_acl = get_branch_acl(dir_acl, "trunk");
    assert(trunk_acl != NULL);

    foo_ace = get_file_on_branch_ace(trunk_acl, "foo");
    assert(foo_ace != NULL);
    bar_ace = get_file_on_branch_ace(trunk_acl, "bar");
    assert(bar_ace != NULL);

    /* set different checkout permissions for files and save them to
       disk */
    set_branch_ace(foo_ace, "vasya", branch_perm_checkout);
    set_branch_ace(foo_ace, "petya", branch_perm_checkin);
    set_branch_ace(bar_ace, "vasya", branch_perm_checkin);
    set_branch_ace(bar_ace, "petya", branch_perm_checkout);

    if (!store_branch_acls(dir_acl)) {
	perror("Storing branch ACLs");
	exit(1);
    }

    /* read those permissions back */
    trunk_acl = get_branch_acl (dir_acl, "trunk");
    assert(trunk_acl != NULL);
    foo_ace = get_file_on_branch_ace(trunk_acl, "foo");
    assert(foo_ace != NULL);
    bar_ace = get_file_on_branch_ace(trunk_acl, "bar");
    assert(bar_ace != NULL);

    assert(branch_permission_granted(foo_ace, "vasya", branch_perm_checkout));
    assert(branch_permission_granted(foo_ace, "petya", branch_perm_checkin));
    assert(branch_permission_granted(bar_ace, "vasya", branch_perm_checkin));
    assert(branch_permission_granted(bar_ace, "petya", branch_perm_checkout));
}

static void
check_persistent_no_perm_acl (void)
{
    DIR_ACL *dir_acl;
    BRANCH_ACL *trunk_acl;
    BRANCH_ACE *quux_ace;

    dir_acl = get_dir_acl("./test-acl", "trunk");
    assert(dir_acl != NULL);

    trunk_acl = get_branch_acl(dir_acl, "trunk");
    assert(trunk_acl != NULL);

    quux_ace = get_file_on_branch_ace(trunk_acl, "quux");
    assert(quux_ace != NULL);

    assert(set_branch_ace(quux_ace, "vasya", branch_perm_none) == branch_perm_none);

    if (!store_branch_acls(dir_acl)) {
	perror("Storing branch ACLs");
	exit(1);
    }

    trunk_acl = get_branch_acl(dir_acl, "trunk");
    assert(trunk_acl != NULL);

    quux_ace = get_file_on_branch_ace(trunk_acl, "quux");
    assert(quux_ace != NULL);

    assert(!branch_permission_granted(quux_ace, "vasya", branch_perm_checkin));
    assert(!branch_permission_granted(quux_ace, "vasya", branch_perm_checkout));
}

int main (void)
{
    check_single_persistent_file_acl();

    check_several_persistent_files_acl();

    check_persistent_no_perm_acl();

    exit(0);
}

