/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2001

   Test-suite for struct stringbuf

*/


#include "stringbuf.h"

#include <assert.h>
#include <stdlib.h>

static void
test_NULL_stringbuf (void)
{
    /* cannot create stringbuf from NULL string */
    struct stringbuf *buf;
    struct stringbuf *buf2;

    buf = new_stringbuf(NULL);
    assert(buf == NULL);

    /* check that duplicating NULL stringbuf works */
    buf2 = dup_stringbuf(NULL);
    assert(buf2 == NULL);

    /* check that freeing NULL stringbuf works */
    free_stringbuf (buf);
}

static void
test_empty_stringbuf (void)
{
    /* check creating new stringbuf from empty string */
    struct stringbuf *buf;
    struct stringbuf *buf2;

    buf = new_stringbuf("");
    assert(buf->buf != NULL);
    assert(buf->len == 0);
    assert(buf->allocated > 0);
    assert(strcmp(buf->buf, "") == 0);

    /* check duplicating empty stringbuf */
    buf2 = dup_stringbuf(buf);
    assert(buf2->buf != NULL);
    assert(buf2->len == 0);
    assert(buf2->allocated > 0);
    assert(strcmp(buf2->buf, "") == 0);

    /* check freeing empty stringbuf */
    assert(free_stringbuf(buf));
    assert(buf != NULL);
    assert(buf->len == 0);
    assert(buf->buf == NULL);
    assert(buf->allocated == 0);

    /* free the duplicated stringbuf */
    assert(free_stringbuf(buf2));
}

static void
test_stringbuf (void)
{
    struct stringbuf *buf;
    struct stringbuf *buf2;

    /* check creating new stringbuf */
    buf = new_stringbuf("foo");

    assert(buf->buf != NULL);
    assert(strlen(buf->buf) == buf->len);
    assert(buf->allocated > buf->len);
    assert(strcmp(buf->buf, "foo") == 0);

    /* check duplicating stringbuf */
    buf2 = dup_stringbuf(buf);
    assert(buf2 != NULL);
    assert(buf->len == buf2->len);
    assert(buf->allocated == buf2->allocated);
    assert(strcmp(buf->buf, buf2->buf) == 0);
    assert(strcmp(buf->buf, "foo") == 0);

    /* check freeing non-empty stringbuf */
    assert(free_stringbuf(buf));
    assert(buf != NULL);
    assert(buf->len == 0);
    assert(buf->buf == NULL);
    assert(buf->allocated == 0);

    assert(free_stringbuf(buf2));
}

static void
test_catc_stringbuf (void)
{
    struct stringbuf *buf;
    struct stringbuf *tmp;
    unsigned int old_allocated;
    unsigned int old_len;

    buf = new_stringbuf("");

    /* check adding a single character to empty stringbuf */
    tmp = catc_stringbuf(buf, 'c');
    assert(tmp == buf);
    assert(buf->len == 1);
    assert(strcmp(buf->buf, "c") == 0);
    assert(buf->allocated > buf->len);

    /* check adding a single character to non-empty stringbuf */
    tmp = catc_stringbuf(buf, 'c');
    assert(tmp == buf);
    assert(buf->len == 2);
    assert(strcmp(buf->buf, "cc") == 0);
    assert(buf->allocated > buf->len);

    /* add characters until there is a need to reallocate  */
    while (buf->allocated > buf->len + 1) {
	struct stringbuf *tmp;
	tmp = catc_stringbuf(buf, 'c');
    }

    /* the reallocation must now happen */
    old_allocated = buf->allocated;
    old_len = buf->len;
    tmp = catc_stringbuf(buf, 'c');

    /* check that reallocation happened */
    assert(tmp == buf);
    assert(buf->allocated > old_allocated);
    assert(buf->len = old_len + 1);

    assert(free_stringbuf(buf));
}


static void
test_cat_stringbuf (void)
{
    struct stringbuf *buf;
    struct stringbuf *tmp;
    unsigned int old_allocated;
    unsigned int old_len;

    /* cannot cat to NULL stringbuf */
    tmp = cat_stringbuf(NULL, "foo");
    assert(tmp == NULL);

    buf = new_stringbuf("");

    /* check catting of NULL string to stringbuf */
    tmp = cat_stringbuf(buf, NULL);
    assert(tmp == buf);
    assert(buf->len == 0);
    assert(strcmp(buf->buf, "") == 0);

    /* check catting of empty string to stringbuf */
    tmp = cat_stringbuf(buf, "");
    assert(tmp == buf);
    assert(buf->len == 0);
    assert(strcmp(buf->buf, "") == 0);

    /* check catting the string to stringbuf */
    tmp = cat_stringbuf(buf, "foo");
    assert(tmp == buf);
    assert(buf->len == 3);
    assert(buf->allocated > buf->len);
    assert(strcmp(buf->buf, "foo") == 0);

    /* add characters until there is a need to reallocate  */
    while (buf->allocated > buf->len + 1) {
	struct stringbuf *tmp;
	tmp = catc_stringbuf(buf, 'c');
    }

    /* check catting the string to stringbuf with reallocation */
    old_len = buf->len;
    old_allocated = buf->allocated;
    tmp = cat_stringbuf(buf, "bar");
    assert(tmp == buf);
    assert(buf->allocated > old_allocated);
    assert(buf->len = old_len + 3);

    assert(free_stringbuf(buf));
}

static void
test_shrink_stringbuf (void)
{
    struct stringbuf *buf;
    struct stringbuf *tmp;

    buf = new_stringbuf("foobar");
    assert(buf != NULL);

    /* cannot shrink to larger size */
    tmp = shrink_stringbuf(buf, 100);
    assert(tmp == NULL);

    tmp = shrink_stringbuf(buf, 3);
    assert(tmp == buf);
    assert(buf->len == 3);
    assert(strcmp(buf->buf, "foo") == 0);
}


int
main(void)
{
    test_NULL_stringbuf();

    test_empty_stringbuf();

    test_stringbuf();

    test_catc_stringbuf();

    test_cat_stringbuf();

    test_shrink_stringbuf();

    exit(0);
}
