
#include "dir-acl.h"
#include "branch-acl.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

static void
check_one_level_traversing (void)
{
    DIR_ACL *topdir_acl;
    BRANCH_ACL *trunk_acl;
    BRANCH_ACE *foo_ace;

    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl != NULL);

    /* set the no-access permissions for alice to top dir */
    assert(set_dir_ace(topdir_acl, "alice", dir_perm_none) == dir_perm_none);

    trunk_acl = get_branch_acl(topdir_acl, "trunk");
    assert(trunk_acl != NULL);

    foo_ace = get_file_on_branch_ace(trunk_acl, "foo");
    assert(foo_ace != NULL);

    /* check that file cannot be accessed by alice at all  */
    assert(!branch_permission_granted(foo_ace, "alice", branch_perm_checkin));
    assert(!branch_permission_granted(foo_ace, "alice", branch_perm_checkout));


    /* set the no-modify permissions for bob to top dir */
    assert(set_dir_ace(topdir_acl, "bob", dir_perm_access) == dir_perm_access);

    trunk_acl = get_branch_acl(topdir_acl, "trunk");
    assert(trunk_acl != NULL);

    foo_ace = get_file_on_branch_ace(trunk_acl, "foo");
    assert(foo_ace != NULL);

    /* check that file can only be checked out by bob, and not checked in */
    assert(branch_permission_granted(foo_ace, "bob", branch_perm_checkout));
    assert(!branch_permission_granted(foo_ace, "bob", branch_perm_checkin));

    /* all others are subject to repository policy */
    set_default_dir_perm(dir_perm_none);
    assert(!branch_permission_granted(foo_ace, "carol", branch_perm_checkout));
    assert(!branch_permission_granted(foo_ace, "carol", branch_perm_checkin));

    set_default_dir_perm(dir_perm_all);
    assert(branch_permission_granted(foo_ace, "carol", branch_perm_checkout));
    assert(branch_permission_granted(foo_ace, "carol", branch_perm_checkin));

    /* store those permissions to disk and verify afterwards */
    if (store_dir_acl(topdir_acl) != 0) {
	perror("Storing dir ACL");
	exit(1);
    }

    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl != NULL);

    trunk_acl = get_branch_acl(topdir_acl, "trunk");
    assert(trunk_acl != NULL);

    foo_ace = get_file_on_branch_ace(trunk_acl, "foo");
    assert(foo_ace != NULL);

    assert(!branch_permission_granted(foo_ace, "alice", branch_perm_checkin));
    assert(!branch_permission_granted(foo_ace, "alice", branch_perm_checkout));
    assert(branch_permission_granted(foo_ace, "bob", branch_perm_checkout));
    assert(!branch_permission_granted(foo_ace, "bob", branch_perm_checkin));

    /* all others are subject to repository policy */
    set_default_dir_perm(dir_perm_all);
    assert(branch_permission_granted(foo_ace, "carol", branch_perm_checkout));
    assert(branch_permission_granted(foo_ace, "carol", branch_perm_checkin));

    set_default_dir_perm(dir_perm_none);
    assert(!branch_permission_granted(foo_ace, "carol", branch_perm_checkout));
    assert(!branch_permission_granted(foo_ace, "carol", branch_perm_checkin));
}


static void
check_two_level_traversing (void)
{
    DIR_ACL *topdir_acl;
    DIR_ACL *sdir_acl;
    DIR_ACL *ssdir_acl;

    set_toplevel_directory("./test-acl");

    topdir_acl = get_dir_acl("./test-acl", "trunk");
    assert(topdir_acl != NULL);

    sdir_acl = get_dir_acl("./test-acl/sdir/", "trunk");
    assert(topdir_acl != NULL);

    ssdir_acl = get_dir_acl("./test-acl/sdir/ssdir/", "trunk");
    assert(ssdir_acl != NULL);

    /* check that alice can't get access neither to sdir/, nor to
       sdir/ssdir/ because she has no access to the top directory */
    assert(!dir_permission_granted(sdir_acl, "alice", dir_perm_access));
    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_access));


    /* check that bob can access both subdirectories, but not to write
       to either of them */
    assert(dir_permission_granted(sdir_acl, "bob", dir_perm_access));
    assert(dir_permission_granted(ssdir_acl, "bob", dir_perm_access));
    assert(!dir_permission_granted(sdir_acl, "bob", dir_perm_modify));
    assert(!dir_permission_granted(ssdir_acl, "bob", dir_perm_modify));

}

static void
remove_acl_files (void)
{
    unlink("./test-acl/ACL");
    unlink("./test-acl/DIRACL");
    unlink("./test-acl/sdir/ACL");
    unlink("./test-acl/sdir/DIRACL");
    unlink("./test-acl/sdir/ssdir/ACL");
    unlink("./test-acl/sdir/ssdir/DIRACL");
}

/* check traversing when 
   - parents don't have associated ACLs;
   - target directory doesn't have one;
*/
static void
check_traversing_case1(void)
{
    DIR_ACL* ssdir_acl;

    remove_acl_files();
    set_toplevel_directory("./test-acl");

    /* check that repository policy is in effect */
    ssdir_acl = get_dir_acl("./test-acl/sdir/ssdir", "trunk");
    assert(ssdir_acl != NULL);

    set_default_dir_perm(dir_perm_all);
    assert(dir_permission_granted(ssdir_acl, "alice", dir_perm_modify));
    assert(dir_permission_granted(ssdir_acl, "alice", dir_perm_access));

    /* set repository policy to none, and check that it works */
    set_default_dir_perm(dir_perm_none);

    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_modify));
    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_access));

    /* set repository policy back to all */
    set_default_dir_perm(dir_perm_all);
}

/* check traversing when 
   - parents don't have associated ACLs;
   - target directory does have one;
*/
static void
check_traversing_case2(void)
{
    DIR_ACL* ssdir_acl;

    remove_acl_files();
    set_toplevel_directory("./test-acl");

    ssdir_acl = get_dir_acl("./test-acl/sdir/ssdir", "trunk");
    assert(ssdir_acl != NULL);

    assert(set_dir_ace(ssdir_acl, "alice", dir_perm_access) == dir_perm_access);

    if (store_dir_acl(ssdir_acl) != 0) {
	perror("Storing dir ACL");
	exit(1);
    }

    /* check that target directory permissions are in effect */
    ssdir_acl = get_dir_acl("./test-acl/sdir/ssdir", "trunk");
    assert(ssdir_acl != NULL);

    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_modify));
    assert(dir_permission_granted(ssdir_acl, "alice", dir_perm_access));

    /* set repository policy to none, and check that it does not matter */
    set_default_dir_perm(dir_perm_none);

    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_modify));
    assert(dir_permission_granted(ssdir_acl, "alice", dir_perm_access));
}


/* check traversing when
   - parents do have associated ACLs;
   - target directory doesn't have one;
*/
static void
check_traversing_case3(void)
{
    DIR_ACL* sdir_acl;
    DIR_ACL* ssdir_acl;

    remove_acl_files();
    set_toplevel_directory("./test-acl");

    sdir_acl = get_dir_acl("./test-acl/sdir", "trunk");
    assert(sdir_acl != NULL);

    assert(set_dir_ace(sdir_acl, "alice", dir_perm_access) == dir_perm_access);

    if (store_dir_acl(sdir_acl) != 0) {
	perror("Storing dir ACL");
	exit(1);
    }

    /* check that parents permissions are in effect */
    ssdir_acl = get_dir_acl("./test-acl/sdir/ssdir", "trunk");
    assert(ssdir_acl != NULL);

    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_modify));
    assert(dir_permission_granted(ssdir_acl, "alice", dir_perm_access));

    /* set repository policy to none, and check that it does not matter */
    set_default_dir_perm(dir_perm_none);

    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_modify));
    assert(dir_permission_granted(ssdir_acl, "alice", dir_perm_access));

    /* set repository policy back to all */
    set_default_dir_perm(dir_perm_all);
}

/* check traversing when
   - parents do have associated ACLs;
   - target directory do have one;
*/
static void
check_traversing_case4(void)
{
    DIR_ACL* sdir_acl;
    DIR_ACL* ssdir_acl;

    remove_acl_files();
    set_toplevel_directory("./test-acl");

    /* set the parent directory permissions */
    sdir_acl = get_dir_acl("./test-acl/sdir", "trunk");
    assert(sdir_acl != NULL);

    assert(set_dir_ace(sdir_acl, "alice", dir_perm_access) == dir_perm_access);

    if (store_dir_acl(sdir_acl) != 0) {
	perror("Storing dir ACL");
	exit(1);
    }

    /* set the target directory permissions */
    ssdir_acl = get_dir_acl("./test-acl/sdir/ssdir", "trunk");
    assert(ssdir_acl != NULL);

    assert(set_dir_ace(ssdir_acl, "alice", dir_perm_none) == dir_perm_none);

    if (store_dir_acl(ssdir_acl) != 0) {
	perror("Storing dir ACL");
	exit(1);
    }

    /* check that both parents and target directory permissions are in effect */
    ssdir_acl = get_dir_acl("./test-acl/sdir/ssdir", "trunk");
    assert(ssdir_acl != NULL);

    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_modify));
    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_access));

    /* set repository policy to none, and check that it does not matter */
    set_default_dir_perm(dir_perm_none);

    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_modify));
    assert(!dir_permission_granted(ssdir_acl, "alice", dir_perm_access));

    /* set repository policy back to all */
    set_default_dir_perm(dir_perm_all);
}

int main (void)
{
    check_one_level_traversing();

    check_two_level_traversing();

    check_traversing_case1();

    check_traversing_case2();

    check_traversing_case3();

    check_traversing_case4();

    exit(0);
}
